// RELATED SOFTWARE: Scilab >= 5.5.0
// PACKAGE  : gui2bitmap : https://atoms.scilab.org/toolboxes/gui2bitmap
//            builds a png | jpg | gif | bmp image from a figure
//            possibly including uicontrols, with|without the window frame
// VERSION  : 1.0 - 2019-08-02
// AUTHOR   : Samuel GOUGEON
// COPYRIGHT: Samuel GOUGEON -  2019
// LICENSE  : CeCILL : http://www.cecill.info/licences/Licence_CeCILL_V2.1-en.txt
//
// DISCLAIMER: This toolbox is provided as is, with NO WARRANTY of any kind.
//             Use it to your own risks & responsability.
//

function gui2bitmap(idFig, filename, withBorders)
    // gui2bitmap(idFig, filename)
    // gui2bitmap(idFig, filename, withBorders)
    //
    // idFig    : Figure's handle. The figure must be not docked.
    //            On linux and MacOS, it is supposed to be not maximized.
    // filename : single string = pathanme of the image file to generate.
    //             The filename extension sets the image encoding.
    //             Supported output encodings: .png .jpg .gif .bmp
    //             Predefined constant paths SCI SCIHOME TMPDIR home..
    //             are supported.
    // withBorders: single boolean: %T to capture the whole figure with
    //              its borders. Default = %F

    fname = "gui2bitmap"
    smode = getscilabmode() 
    if and(smode <> ["STD" "NW"]) then
        msg = _("%s can''t work in %s mode. Aborted.\n")
        error(msprintf(msg, smode))
    end

    // PRIVATE FUNCTIONS
    // -----------------
    function r = is_maximized(idFig)
        // Windows only. Works wherever the taskbar is located
        fs = idFig.figure_size
        [xmax, ymax] = (getsystemmetrics("SM_CXMAXIMIZED"), ..
                        getsystemmetrics("SM_CYMAXIMIZED"));
        r = and(idFig.figure_size == [xmax ymax]);
    endfunction
    function yn = isdocked(fh)
        // For Scilab 5.5.2 (is included & named isDocked() in Scilab 6.0
        // http://fileexchange.scilab.org/toolboxes/360000
        // Shared internal function used by clf(), fchamp(), etc.
        //
        // fh: matrix of figures handles
        // yn: matrix of booleans
        //
        // yn(i) is %T if fh(i) is docked, %F otherwise
    
        sf = size(fh)
        fh = fh(:)
        fs = matrix(fh.figure_size,2,-1)'
        as = matrix(fh.axes_size,2,-1)'
    
        yn = (fh.dockable=="on" & ((fs(:,1)-as(:,1)) > 20)')
        // A test on vertical dimensions is more complicated, due to switchable
        // menubar, toolbar and infobar.
        yn = matrix(yn, sf)
    endfunction


    // CHECK INPUT ARGUMENTS
    // ---------------------
    rhs = argn(2);
    if rhs < 2 | rhs > 3  then
        msg = _("%s: Wrong number of input arguments: %d or %d expected.\n")
        error(msprintf(msg, fname,2,3))
    end
    // idFig
    if ~isdef("idFig","l") | type(idFig)<>9 | or(idFig.type<>"Figure") then
        msg = _("%s: Argument #%d: Graphic handle of type ''%s'' expected.\n")
        error(msprintf(msg, fname,1,"Figure"))
    else
        idFig = idFig(1)
    end
    if  isdocked(idFig) then
        msg = _("gui2bitmap","%s: The figure #%d is docked => Processing impossible.\n")
        error(msprintf(msg, fname, idFig.figure_id))
    end
    // filename
    if ~isdef("filename","l") | type(filename)<>10 then
        msg = _("gui2bitmap", "%s: Argument #%d: Text expected.\n")
        error(msprintf(msg, fname,2))
    end
    filename = pathconvert(filename(1),%f) // support to SCI SCIHOME TMPDIR home..
    dname = fileparts(filename)
    if dname <> ""
        if ~isdir(dname) then
            mkdir(dname)
        end
    end
    // withBorders
    tmp = isdef("withBorders","l")
    if ~tmp then
        withBorders = %f
    end
    if type(withBorders)<>4  then
        msg = _("%s: Argument #%d: Boolean expected.\n")
        error(msprintf(msg, fname,3))
    end
    withBorders = withBorders(1)


    // PROCESSING
    // ==========
    // Brings the GUI window on the front
    idFig.visible = "on";

    is_max = getos()=="Windows" & is_maximized(idFig);
    fp = idFig.figure_position;
    fs = idFig.figure_size;
    as = idFig.axes_size;

    // Insure that the figure is not clipped out of screen on the left and top
    fp_old = fp;
    if ~is_max then  // When maximized, the left and top borders may be at < 0
        if fp(1)<0 then
            idFig.figure_position(1) = 0;
            fp(1) = 0
        end
        if fp(2)<0 then
            idFig.figure_position(2) = 0;
            fp(2) = 0;
        end
    end

    // Compute the position and sizes of the area to grab
    border_width = (fs(1)-as(1))/2;     // always correct
    header_height = fs(2) - as(2) - border_width;
    // infobar_height = ?
    if ~withBorders then
        if idFig.infobar_visible=="off" then
            infobar_height = 0;
        else
            tmp1 = idFig.figure_size;
            idFig.infobar_visible = "off";
            tmp2 = idFig.figure_size;
            idFig.infobar_visible = "on";
            infobar_height = tmp1(2) - tmp2(2);
        end
        header_height = header_height - infobar_height
    end

    if withBorders then
        // Testée : OK (maximisée ou non, avec infobar ou non)
        [x, y, w, h] = (fp(1), fp(2), fs(1), fs(2));
        if is_max // taskbar @ bottom
            h = h - border_width
            // left, right and bottom borders can't be grabbed
        end
    else
        [x, y, w, h] = (fp(1)+border_width, fp(2)+header_height, as(1), as(2));
    end

    // Grabs the figure into an image
    jimport java.awt.Rectangle;
    jimport java.awt.Robot;
    jimport java.awt.image.BufferedImage;
    jimport java.io.File;
    jimport javax.imageio.ImageIO;
    bounds = Rectangle.new(x, y, w, h);
    imtype = strsubst(fileparts(filename,"extension"),".","");
    rob = jnewInstance(Robot);
    ImageIO.write(rob.createScreenCapture(bounds), imtype, File.new(filename));

    // Finish
    jremove bounds rob BufferedImage File ImageIO Rectangle Robot
    idFig.figure_position = fp_old;
endfunction
